"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEntityReducer = createEntityReducer;

var _modify = require("./modify");

var _remove = require("./remove");

var _upsert = require("./upsert");

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function isActionInRoutines(routines, action) {
  return routines && routines.some(function (routine) {
    return routine.sync ? routine.TRIGGER === action.type : routine.SUCCESS === action.type;
  });
}

function isActionReflected(on, action) {
  return on && Object.keys(on).indexOf(action.type) !== -1;
}

function isActionRecognized(collection, action) {
  return collection instanceof Array ? isActionInRoutines(collection, action) : isActionReflected(collection, action);
}

function empty() {
  return [];
}

function reduceModifyReducer(reducers) {
  return function (acc, key) {
    return _objectSpread({}, acc, _defineProperty({}, key, (0, _modify.createModifyReducer)(reducers[key])));
  };
}

function getModifyReducers(reducers) {
  return reducers ? Object.keys(reducers).reduce(reduceModifyReducer(reducers), {}) : null;
}

function createEntityReducer(_ref) {
  var clearedBy = _ref.clearedBy,
      deletedBy = _ref.deletedBy,
      on = _ref.on,
      collectionReducers = _ref.collectionReducers,
      providedBy = _ref.providedBy,
      reducerOptions = _objectWithoutProperties(_ref, ["clearedBy", "deletedBy", "on", "collectionReducers", "providedBy"]);

  var itemReducers = getModifyReducers(on);

  function getReducerSequence(action) {
    return [isActionRecognized(clearedBy, action) && empty, isActionRecognized(collectionReducers, action) && collectionReducers[action.type], isActionRecognized(providedBy, action) && _upsert.upsert, isActionRecognized(itemReducers, action) && itemReducers[action.type], isActionRecognized(deletedBy, action) && _remove.remove].filter(function (item) {
      return item;
    });
  }

  return function entityReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var action = arguments.length > 1 ? arguments[1] : undefined;
    return getReducerSequence(action).reduce(function (acc, reducer) {
      return reducer(acc, action, reducerOptions);
    }, state);
  };
}