"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.operations = void 0;

var _constants = require("../constants");

var _operationMap;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var initialState = {
  error: null,
  initialized: false,
  loading: false
};

function filterStage(actionType) {
  return function (stage) {
    return actionType.indexOf(stage) === actionType.length - stage.length;
  };
}

function getStage(actionType) {
  return _constants.STAGES_ASYNC.find(filterStage(actionType));
}

function getBaseName(actionType, stage) {
  return actionType.substr(0, actionType.length - stage.length - 1);
}

function reduceOperation(state, action, baseName, reducer) {
  return _objectSpread({}, state, _defineProperty({}, baseName, reducer(state[baseName] || initialState, action)));
}

function startLoading(state) {
  return _objectSpread({}, state, {
    error: null,
    loading: true
  });
}

function stopLoading(state) {
  return _objectSpread({}, state, {
    loading: false
  });
}

function initialize(state) {
  return _objectSpread({}, state, {
    initialized: true
  });
}

function fail(state, action) {
  return _objectSpread({}, state, {
    error: action.payload
  });
}

var operationMap = (_operationMap = {}, _defineProperty(_operationMap, _constants.STAGE_FAILURE, fail), _defineProperty(_operationMap, _constants.STAGE_FULFILL, stopLoading), _defineProperty(_operationMap, _constants.STAGE_REQUEST, startLoading), _defineProperty(_operationMap, _constants.STAGE_SUCCESS, initialize), _operationMap);

function requireStage(reducer) {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments.length > 1 ? arguments[1] : undefined;
    var stage = action.type && getStage(action.type);
    return stage ? reducer(state, action, stage) : state;
  };
}

function requireStageReducer(reducer) {
  return function (state, action, stage) {
    var stageReducer = operationMap[stage];
    return stageReducer ? reducer(state, action, stage, stageReducer) : state;
  };
}

var operations = requireStage(requireStageReducer(function (state, action, stage, stageReducer) {
  return reduceOperation(state, action, getBaseName(action.type, stage), stageReducer);
}));
exports.operations = operations;