"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRoutine = createRoutine;
exports.createSyncRoutine = createSyncRoutine;
exports.createAsyncRoutine = createAsyncRoutine;
exports.createEntityRoutines = createEntityRoutines;

var _actions = require("./actions");

var _selectors = require("./selectors");

var _camelcase = _interopRequireDefault(require("camelcase"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createRoutine(baseName) {
  var routine = (0, _actions.createActionCreator)(baseName);
  routine.routineName = baseName;
  return routine;
}

function createSyncRoutine(baseName) {
  var routine = createRoutine(baseName);
  routine.trigger = routine;
  routine.TRIGGER = baseName;
  routine.sync = true;
  return routine;
}

function createAsyncRoutine(baseName) {
  var TRIGGER = (0, _actions.composeActionType)(baseName, 'TRIGGER');
  var routine = createRoutine(TRIGGER);
  routine.trigger = routine;
  routine.TRIGGER = TRIGGER;
  routine.FAILURE = (0, _actions.composeActionType)(baseName, 'FAILURE');
  routine.failure = (0, _actions.createActionCreator)(routine.FAILURE);
  routine.FULFILL = (0, _actions.composeActionType)(baseName, 'FULFILL');
  routine.fulfill = (0, _actions.createActionCreator)(routine.FULFILL);
  routine.REQUEST = (0, _actions.composeActionType)(baseName, 'REQUEST');
  routine.request = (0, _actions.createActionCreator)(routine.REQUEST);
  routine.SUCCESS = (0, _actions.composeActionType)(baseName, 'SUCCESS');
  routine.success = (0, _actions.createActionCreator)(routine.SUCCESS);
  routine.getState = (0, _selectors.createOperationSelector)(baseName);
  routine.getError = (0, _selectors.getOperationProp)(routine.getState, 'error');
  routine.isInitialized = (0, _selectors.getOperationFlag)(routine.getState, 'initialized');
  routine.isLoading = (0, _selectors.getOperationFlag)(routine.getState, 'loading');
  return routine;
}

function reduceRoutinesToDict(entity, sync) {
  return function (acc, routine) {
    var routineName = routine.toUpperCase();
    var baseName = (0, _actions.composeActionType)(entity, routineName);

    var result = _objectSpread({}, acc, _defineProperty({}, (0, _camelcase["default"])(routine), sync ? createSyncRoutine(baseName) : createAsyncRoutine(baseName)));

    if (sync) {
      result[routineName] = baseName;
    }

    return result;
  };
}

function createEntityRoutines(entity, routines, sync) {
  return routines.reduce(reduceRoutinesToDict(entity, sync), {});
}