"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEntityStore = createEntityStore;

var _Configurable2 = require("./Configurable");

var _reducers = require("./reducers");

var _reselect = require("reselect");

var _routines = require("./routines");

var _selectors = require("./selectors");

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function flat(arrays) {
  return arrays.reduce(function (acc, arr) {
    return acc.concat(arr);
  }, []);
}

function mergeArrays() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return flat(args).filter(function (item) {
    return item;
  }).filter(_reducers.filterUnique);
}

var EntityStore =
/*#__PURE__*/
function (_Configurable) {
  _inherits(EntityStore, _Configurable);

  function EntityStore(name, config) {
    var _this;

    _classCallCheck(this, EntityStore);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(EntityStore).call(this, _objectSpread({}, config, {
      name: name
    })));

    _this.bind('findItem');

    _this.bind('getAll');

    _this.bind('getCollection');

    _this.bind('getSize');

    _this.bind('isEmpty');

    _this.name = name;
    _this.getObject = (0, _reselect.createSelector)(_this.getCollection, _selectors.getFirstSelectorArg, _this.findItem);
    _this.getProp = (0, _reselect.createSelector)(_this.getObject, _selectors.getSecondSelectorArg, _selectors.getProp);
    _this.getFlag = (0, _reselect.createSelector)(_this.getProp, _selectors.getFlag);
    return _this;
  }

  _createClass(EntityStore, [{
    key: "initialize",
    value: function initialize() {
      this.createActions();
      this.createReducer();
    }
  }, {
    key: "createActions",
    value: function createActions() {
      this.clear = (0, _routines.createSyncRoutine)("".concat(this.name.toUpperCase(), "/CLEAR"));
      this.configure({
        clearedBy: mergeArrays([this.clear], this.config.clearedBy)
      });
    }
  }, {
    key: "createReducer",
    value: function createReducer() {
      this.reducer = (0, _reducers.createEntityReducer)(this.config);
    }
  }, {
    key: "findItem",
    value: function findItem(items, ident) {
      return items.find((0, _reducers.filterItem)(this.config, ident)) || null;
    }
  }, {
    key: "getCollection",
    value: function getCollection(state) {
      return state.entities[this.name];
    }
  }, {
    key: "getAll",
    value: function getAll(state) {
      return this.getCollection(state);
    }
  }, {
    key: "createFindSelector",
    value: function createFindSelector(getIdent) {
      return (0, _reselect.createSelector)(this.getCollection, getIdent, this.findItem);
    }
  }, {
    key: "getSize",
    value: function getSize(state) {
      return this.getAll(state).length;
    }
  }, {
    key: "isEmpty",
    value: function isEmpty(state) {
      return this.getSize(state) === 0;
    }
  }, {
    key: "toString",
    value: function toString() {
      return "EntityStore(".concat(this.name, ")");
    }
  }]);

  return EntityStore;
}(_Configurable2.Configurable);

function createEntityStore(collectionName, config) {
  return new EntityStore(collectionName, config);
}